;-----------------------------------------------------------;
;                                                           ;
; Alien Breed IV: Final Assault                             ;
; Miscellaneous Routines                                    ;
;                                                           ;
;-----------------------------------------------------------;

;------------------------------------------------
; getTile - Get a tile given the exact coordinates
;
; Input:    A = X
;           L = Y
; Output:   A = Tile
;           HL => Tile
;------------------------------------------------
getTile:
        push    af
        ld      a,l
        call    divAby8
        ld      l,a
        ld      a,(width)
        ld      h,a
        inc     h
        bcall(_htimesl)
        GETTEMP(de)
        add     hl,de
        pop     af
        call    divAby8
        ld      e,a
        ld      d,0
        add     hl,de
        ld      a,(hl)
        ret

;------------------------------------------------
; getLetter - Get a capital letter
;
; Input:    None
; Output:   A = ASCII character
;------------------------------------------------
getLetter:
        call    waitKey
        dec     a
        ld      e,a
        ld      d,0
        ld      hl,letterTable
        add     hl,de
        ld      a,(hl)
        cp      '.'
        jr      z,getLetter
        ret

;------------------------------------------------
; getTileSave - Get a tile given the exact coordinates, save coordinates
;
; Input:    A = X
;           L = Y
; Output:   A = Tile
;           H = X
;           L = Y
;------------------------------------------------
getTileSave:
        ld      h,a
        push    hl
        call    getTile
        pop     hl
        ret

;------------------------------------------------
; checkOnScreen - Check a point to see if it's on-screen
;
; Input:    A = X
;           L = Y
; Output:   A = X
;           H = X
;           L = Y
;           CA = Point is on the viewing screen
;------------------------------------------------
checkOnScreen:
        ld      h,a
        push    hl
        ld      h,1
        ld      (collideObject1+2),hl
        ld      l,a
        ld      (collideObject1),hl
        ld      hl,(scrX)
        ld      h,96
        ld      (collideObject2),hl
        ld      hl,(scrY)
        ld      h,64
        ld      (collideObject2+2),hl
        call    checkCollision
        pop     hl
        ld      a,h
        ret

;------------------------------------------------
; putTile - Put a tile into a level and draw it on the screen
;
; Input:    HL => Tile location in level
;           A = Tile to put
; Output:   HL => Tile location in level
;------------------------------------------------
putTile:
        push    hl
        ld      (hl),a
        push    af
        call    getXYFromHL
        ld      b,a
        ld      a,l
        TIMES8(a)
        ld      hl,scrY
        sub     (hl)
        ld      c,a
        ld      a,b
        TIMES8(a)
        ld      hl,scrX
        sub     (hl)
        ld      b,a
        push    bc
        ld      hl,sprWhiteSpace
        call    putClippedMaskedSprite
        pop     bc
        pop     af
        ld      l,a
        ld      h,0
        TIMES8(hl)
        ld      de,startTiles
        add     hl,de
        call    putClippedSprite
        pop     hl
        ret

;------------------------------------------------
; getXYFromHL - Get X & Y tile coordinates from a pointer to a tile in memory
;
; Input:    HL => Tile in level data
; Output:   A = X (block value)
;           L = Y (block value)
;------------------------------------------------
getXYFromHL:
        GETTEMP(de)
        or      a
        sbc     hl,de
        ld      a,(width)
        inc     a
        bcall(_divhlbya)
        ret

;------------------------------------------------
; getExactXYFromHL - Get X & Y coordinates form a pointer to a tile in memory
;
; Input:    HL => Tile in level data
; Output:   A = X
;           H = X
;           L = Y
;------------------------------------------------
getExactXYFromHL:
        call    getXYFromHL
        ld      h,0
        TIMES8(hl)
        TIMES8(a)
        ld      h,a
        ret

;------------------------------------------------
; saveVideo - Save gbuf to videoBuffer
;
; Input:    None
; Output:   None
;------------------------------------------------
saveVideo:
        ld      hl,gbuf
        ld      de,videoBuffer
#ifdef TI86
        ld      bc,1024
#else
        ld      bc,768
#endif
        ldir
        ret

;------------------------------------------------
; restoreVideo - Restore videoBuffer to gbuf
;
; Input:    None
; Output:   None
;------------------------------------------------
restoreVideo:
        ld      hl,videoBuffer
        ld      de,gbuf
#ifdef TI86
        ld      bc,1024
#else
        ld      bc,768
#endif
        ldir
        ret

;------------------------------------------------
; lowerObjectives - Lower objectives counter
;
; Input:    None
; Output:   None
;------------------------------------------------
lowerObjectives:
        ld      hl,objectives
        ld      a,(hl)
        cp      2
        ret     c
        dec     (hl)
        ret

;------------------------------------------------
; invertBox - Do a selection box on gbuf
;
; Input:    HL => Where to start inverting
;           B = Height of box
;           C = Width of box (in bytes)
; Output:   None
;------------------------------------------------
invertBox:
        push    hl
        ld      d,c
invertBoxLoop:
        ld      a,(hl)
        cpl
        ld      (hl),a
        inc     hl
        dec     d
        jr      nz,invertBoxLoop
        pop     hl
#ifdef TI86
        ld      de,16
#else
        ld      de,12
#endif
        add     hl,de
        djnz    invertBox
        ret

;------------------------------------------------
; waitKey - Wait for a key press
;
; Input:    None
; Output:   A = Key pressed (_getcsc style)
;------------------------------------------------
waitKey:
        ei
        res     apdrunning,(iy+apdflags)
wKeyLoop:
        halt
        bcall(GET_KEY)
        or      a
        jr      z,wKeyLoop
        set     apdrunning,(iy+apdflags)
        ret

;------------------------------------------------
; waitKeyA - Wait for a key press, excluding arrow pad
;
; Input:    None
; Output:   A = Key pressed (_getcsc style)
;------------------------------------------------
waitKeyA:
        call    waitKey
        cp      5
        jr      c,waitKeyA
        ret

;------------------------------------------------
; divAby8 - Divide A by 8
;
; Input:    A = Number to divide
; Output:   A = A / 8
;------------------------------------------------
divAby8:
        srl     a
        srl     a
        srl     a
        ret

;------------------------------------------------
; setShowHL - Show HL in small font at specified position
;
; Input:    HL = Value to show
;           DE = penrow, pencol
; Output:   None
;------------------------------------------------
setShowHL:
        ld      (pencol),de

;------------------------------------------------
; showHL - Show HL in small font at cursor position
;
; Input:    HL = Value to show
; Output:   None
;------------------------------------------------
showHL:
        ld      de,stringBuffer+5
        ld      b,5
        xor     a
        ld      (de),a
showHLLoop:
        dec     de
        bcall(_divHLby10)
        add     a,'0'
        ld      (de),a
        djnz    showHLLoop
        ex      de,hl
        bcall(_vputs)
        ret

;------------------------------------------------
; MirageOS routines that need to be written for Ion/TI-86 versions
;------------------------------------------------
#ifdef MIRAGE

vPuts           = setVPuts                      ; Redirection to MirageOS library function

#else                                           ; Ion/TI-86 versions need their own copy of the function

;------------------------------------------------
; directIn - Read keypresses from a key group
;
; Input:    A = Key group code
; Output:   A = Keypresses from specified group
;------------------------------------------------
directIn:
        ld      b,a
        ld      a,$FF
        out     (1),a
        ld      a,b
        out     (1),a
        in      a,(1)
        ret

;------------------------------------------------
; vPuts - Same as _vputs but sets cursor position
;
; Input:    DE = Value to store to (pencol)
;           HL => String to display
; Output:   HL => Next string
;------------------------------------------------
vPuts:
        ld      (pencol),de
        bcall(_vputs)
        ret

#endif

#ifdef TI86                                     ; Routines not on the TI-86

;------------------------------------------------
; videoCenterCopy - For TI-86 only, shift entire gbuf to the right 16 pixels then to ionFastCopy
;
; Input:    None
; Output:   None
;------------------------------------------------
videoCenterCopy:
        ld      hl,gbuf+1021
        ld      de,gbuf+1023
        ld      bc,1022
        lddr
        inc     hl
        ld      (hl),0
        inc     hl
        ld      (hl),0

;------------------------------------------------
; ionFastCopy - Copy gbuf to lcd
;
; Input:    None
; Output:   None
;------------------------------------------------
ionFastCopy:
        ld      hl,gbuf
        ld      de,videoMem
        ld      bc,1024
        ldir
        ret

;------------------------------------------------
; ionGetPixel - Get pixel information
;
; Input:    E = Y
;           A = X
; Output:   A = Bit mask
;           HL => Byte where pixel is on gbuf
;------------------------------------------------
ionGetPixel:
        ld      d,$00
        ld      h,d
        ld      l,e
        add     hl,hl
        add     hl,hl
        add     hl,hl
        add     hl,hl
        ld      de,gbuf
        add     hl,de
        ld      b,$00
        ld      c,a
        and     %00000111
        srl     c
        srl     c
        srl     c
        add     hl,bc
        ld      b,a
        inc     b
        ld      a,%00000001
getPixelLoop:
        rrca
        djnz    getPixelLoop
        ret

;------------------------------------------------
; ionRandom - Generate a random number
;
; Input:    B = Upper bound
; Output:   A = Random number, 0 <= A < B
;------------------------------------------------
ionRandom:
        push    hl
        push    de
        ld      hl,(randData)
        ld      a,r
        ld      d,a
        ld      e,(hl)
        add     hl,de
        add     a,l
        xor     h
        ld      (randData),hl
        sbc     hl,hl
        ld      e,a
        ld      d,h
randomLoop:
        add     hl,de
        djnz    randomLoop
        ld      a,h
        pop     de
        pop     hl
        ret
randData        = _p_imathptr1

;------------------------------------------------
; _disphl - Display HL in large font
;
; Input:    HL = Number to display
; Output:   None
;------------------------------------------------
_disphl:
        push    af
        xor     a
        call    _dispahl
        pop     af
        ret

;------------------------------------------------
; _grbufclr - Clear gbuf
;
; Input:    None
; Output:   None
;------------------------------------------------
_grbufclr:
        ld      hl,gbuf
        ld      de,gbuf+1
        ld      bc,1023
        ld      (hl),0
        ldir
        ret

;------------------------------------------------
; ionDetect - Find a program using a search string
;
; Authors:  Joe Wingbermuehle (http://joewing.calc.org)
;           Modified by James Vernon for TI-86
; Input:    HL => Place in VAT to start searching from
;           IX => Search string
; Output:   DE => Place stopped + 1
;           HL => Program data (after search string)
;           Z = Found
;------------------------------------------------
ionDetect:
        call    _ram_page_7                     ; Load RAM page 7 (where VAT is stored)
        ld      de,(_ptemp_end+1)               ; DE => End of VAT
        call    _cphlde                         ; Are we at the end of the VAT?
        ld      a,(hl)                          ; A = Data type
        jr      nz,detectContinue               ; If not at the end of the VAT, check this entry
        call    _ram_page_1
        inc     a                               ; NZ
        ret                                     ; Leave
detectContinue:
        push    hl                              ; Save ptr to VAT entry
        cp      $0C                             ; Is it a string?
        jr      nz,detectSkip                   ; If not, try next entry
        dec     hl
        ld      e,(hl)
        dec     hl
        ld      d,(hl)
        dec     hl
        ld      a,(hl)
        ex      de,hl
        call    _ram_page_1
        ld      (realFilePtr),hl
        ld      (realFilePtr+2),a
        call    _set_abs_src_addr
        ld      a,1
        ld      hl,tempProgStorage-$8000
        call    _set_abs_dest_addr
        xor     a
        ld      hl,TEMP_SIZE
        call    _set_mm_num_bytes
        call    _mm_ldir
        call    _ram_page_1
        ld      hl,tempProgStorage+2
        push    ix
        pop     de
detectCheck:
        ld      a,(de)
        or      a
        jr      z,detectFound
        cp      (hl)
        inc     de
        inc     hl
        jr      z,detectCheck
detectNoGood:
        call    _ram_page_7
detectSkip:
        pop     hl
        dec     hl
        dec     hl
        dec     hl
        dec     hl
        dec     hl
        ld      b,(hl)
        dec     hl
detectNameLoop2:
        dec     hl
        djnz    detectNameLoop2
        jr      ionDetect
detectFound:
        ex      (sp),hl
        call    _ram_page_7
        dec     hl
        dec     hl
        dec     hl
        dec     hl
        dec     hl
        ld      b,(hl)
        dec     hl
detectNameLoop1:
        dec     hl
        djnz    detectNameLoop1
        pop     de
        ex      de,hl
        call    _ram_page_1
        xor     a
        ret

#endif

.end
